/**
 * @file
 * Drop2S3 javascript functions.
 */

function Drop2S3(context, instanceID, instanceSettingsJSON) {

  'use strict';

  this.init(context, instanceID, instanceSettingsJSON);
}

(function ($, Drupal) {

  'use strict';

  Drop2S3.prototype = {
    evaporate: null,
    // Init.
    init: function (context, instanceID, instanceSettingsJSON) {
      // Load general settings.
      this.settings = Drupal.settings.Drop2S3;
      // Load instance settings.
      this.instanceSettings = JSON.parse(instanceSettingsJSON);
      // Value store field.
      this.valueField = $('input#' + instanceID, context);
      // Initiate Evaporate JS.
      this.initEvaporateJS();
      // Initiate Dropzone JS.
      this.initDropzoneJS();
      // Bind close error.
      this.bindCloseError();
      // Adding data element.
      this.valueField.data('Drop2S3', this);
    },
    // Initiate Evaporate JS.
    initEvaporateJS: function () {
      var scope = this;
      this.evaporate = new Evaporate({
        signerUrl: Drupal.settings.basePath + this.settings.signerUrl,
        aws_key: scope.settings.accessKey,
        bucket: scope.settings.BucketName,
        logging: scope.settings.consoleLogging,
        progressIntervalMS: scope.instanceSettings.progress_interval
      });
    },
    // Initiate Dropzone JS.
    initDropzoneJS: function () {
      var scope = this;
      var config;
      var customConfig;
      var mergedConfig;
      config = {
        url: Drupal.settings.basePath,
        previewTemplate: '<div class="dz-preview dz-file-preview"></div>',
        maxFiles: 1,
        init: function () {
          this.on('addedfile', scope.addedFileDropzone.bind(scope));
          this.on('sending', scope.sendingDropzone.bind(scope));
          this.on('error', scope.errorDropzone.bind(scope));
        }
      };
      // Custom settings.
      if (typeof scope.instanceSettings.dropzone_custom_configuration !== 'undefined' &&
        !$.isEmptyObject(scope.instanceSettings.dropzone_custom_configuration)) {
        customConfig = JSON.parse(scope.instanceSettings.dropzone_custom_configuration);
      }
      this.droparea = this.valueField.siblings('.js-drop2s3-dropzone');
      // Merge configuration.
      mergedConfig = $.extend(config, customConfig);
      // Create Dropzone instance.
      this.dropzone = new Dropzone(this.droparea.get(0), mergedConfig);
    },
    // Dropzone JS event.
    addedFileDropzone: function (file) {
      this.closeError();
      this.clearStatusBar();
    },
    // Dropzone JS event.
    sendingDropzone: function (file) {
      if (file.accepted) {
        this.addEvaporate(file);
      }
      this.dropzone.removeAllFiles(true);
    },
    // Dropzone JS event.
    errorDropzone: function (file, errorMessage) {
      if (file.status === 'error') {
        this.showError(errorMessage);
      }
    },
    // Drop2s3 span close error.
    bindCloseError: function () {
      var scope = this;
      $('.upload-error .close', scope.droparea).bind('click', function (e) {
        e.stopPropagation();
        scope.closeError();
      });
    },
    // Drop2s3 show error.
    showError: function (errorMessage) {
      $('.upload-error .message', this.droparea).text(errorMessage);
      $('.upload-error', this.droparea).show();
    },
    // Drop2s3 close error.
    closeError: function () {
      $('.upload-error .message', this.droparea).text('');
      $('.upload-error', this.droparea).hide();
    },
    // Upload File.
    addEvaporate: function (file) {
      var scope = this;
      this.started = false;

      var xAmzHeadersAtInitiate = {};
      if (this.settings.encryption) {
        xAmzHeadersAtInitiate['x-amz-server-side-encryption'] = this.settings.encryption;
      }

      var filename = this.getFilename(file.name);

      // Send file to S3 by Evaporate.
      this.evaporateId = scope.evaporate.add({
        name: this.settings.rootFolder + filename,
        file: file,
        contentType: file.type,
        xAmzHeadersAtInitiate: xAmzHeadersAtInitiate,
        notSignedHeadersAtInitiate: {
          'Content-Disposition': 'filename="' + file.name + '"'
        },
        started: function () {
          scope.startedEvaporate();
        },
        error: function (errorMessage) {
          scope.errorEvaporate(errorMessage);
        },
        complete: function (response, awsObjectKey) {
          scope.completeEvaporate(response, awsObjectKey);
        },
        progress: function (progressStatus) {
          scope.progressEvaporate(progressStatus);
        }
      });
    },
    getFilename: function (filename) {
      var rename = this.valueField.triggerHandler('drop2s3.filename.evaporate', [this, filename]);
      if (typeof rename === 'undefined') {
        return filename;
      }
      return rename;
    },
    // Evaporate JS started upload file event.
    startedEvaporate: function () {
      // Trigger started Evaporate.
      this.valueField.trigger('drop2s3.started.evaporate', [this]);
      this.started = new Date().getTime();
      this.droparea.addClass('uploading');
      var hms = $('.hms-placeholder', this.droparea);
      if (hms) {
        hms.text('');
      }
    },
    // Evaporate JS progress upload file event.
    progressEvaporate: function (progressStatus) {
      // Trigger progress Evaporate.
      this.valueField.trigger('drop2s3.progress.evaporate', [this]);
      // Time spent.
      var timeSpent = new Date().getTime() - this.started;
      // Remaining seconds.
      var secondsRemaining = Math.round(((timeSpent / progressStatus) - timeSpent) / 1000);
      // Convert seconds remaining to hour, minutes and seconds.
      var hms = this.secondsToHMS(secondsRemaining);
      if (hms) {
        $('.hms-placeholder', this.droparea).text(hms);
      }
      // Update progress bar.
      var progressPercentage = Math.floor(progressStatus * 100);
      $('.status-bar', this.droparea).css('width', progressPercentage + '%');
    },
    // Evaporate JS event.
    completeEvaporate: function (response, awsObjectKey) {
      this.clearStatusBar();
      this.droparea.removeClass('empty');
      this.droparea.removeClass('uploading').addClass('uploaded');
      var filePath = decodeURIComponent(awsObjectKey);
      this.valueField.val(filePath);
      $('.uploaded-content .file-path', this.droparea).text(filePath);
      if (this.instanceSettings.s3_auto_presigned_url) {
        this.presignedUrl(filePath);
      }
      else {
        this.valueField.trigger('drop2s3.complete.evaporate', [this, filePath, null]);
      }
    },
    // Evaporate JS upload error file error event.
    errorEvaporate: function (errorMessage) {
      this.clearStatusBar();
      this.droparea.removeClass('uploading');
      this.showError(errorMessage);
    },
    // Convert seconds to HMS format.
    secondsToHMS: function (s) {
      var scope = this;
      if (s > 1) {
        var h = Math.floor(s / 3600);
        s -= h * 3600;
        var m = Math.floor(s / 60);
        s -= m * 60;
        var hours = '';
        if (h > 0 && scope.isNumeric(h)) {
          hours = (h === 1) ? h + Drupal.t(' hour ') : h + Drupal.t(' hours ');
        }
        var mins = '';
        if (m > 0 && scope.isNumeric(m)) {
          hours = (m === 1) ? m + Drupal.t(' min ') : m + Drupal.t(' mins ');
        }
        var secs = '';
        if (s > 0 && scope.isNumeric(s)) {
          secs = (s === 1) ? s + Drupal.t(' sec ') : s + Drupal.t(' secs ');
        }
        return hours + mins + secs;
      }
      return false;
    },
    // Check if the number is numeric.
    isNumeric: function (n) {
      return !isNaN(parseFloat(n)) && isFinite(n);
    },

    presignedUrl: function (filePath) {
      var scope = this;
      $.ajax({
        url: Drupal.settings.basePath + this.settings.presignedUrl,
        type: 'GET',
        data: {
          file_path: filePath
        },
        success: function (urlSigned) {
          scope.valueField.trigger('drop2s3.complete.evaporate', [scope, filePath, urlSigned]);
        }
      });
    },

    clearStatusBar: function () {
      $('.status-bar', this.droparea).css('width', '0%');
    }
  };

  // Behavior to initialize Drop2S3 instances.
  Drupal.behaviors.Drop2S3 = {
    attach: function (context) {
      if (typeof Drupal.settings.Drop2S3.instances === 'undefined') {
        return;
      }
      // Disable Dropzone auto discover.
      Dropzone.autoDiscover = false;
      // Disable Dropzone to upload to our server.
      Dropzone.prototype.submitRequest = function () {};
      // Initiate Drop2S3 instances.
      $.each(Drupal.settings.Drop2S3.instances, function (instanceID, instanceSettingsJSON) {
        $('input#' + instanceID).once('drop2s3', function () {
          new Drop2S3(context, instanceID, instanceSettingsJSON);
        });
      });
    }
  };
})(jQuery, Drupal);
